@@ -55,7 +55,7 @@ import org.apache.tomcat.util.net.URL;
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Revision: 784606 $ $Date: 2009-06-14 20:23:34 +0100 (Sun, 14 Jun 2009) $
+ * @version $Revision: 620028 $ $Date: 2008-02-08 16:15:48 -0700 (Fri, 08 Feb 2008) $
  */
 
 public class CoyoteResponse
@@ -802,7 +802,7 @@ public class CoyoteResponse
         ServerCookie.appendCookieValue
             (sb, cookie.getVersion(), cookie.getName(), cookie.getValue(),
              cookie.getPath(), cookie.getDomain(), cookie.getComment(), 
-             cookie.getMaxAge(), cookie.getSecure(), false);
+             cookie.getMaxAge(), cookie.getSecure());
         // if we reached here, no exception, cookie is valid
         // the header name is Set-Cookie for both "old" and v.1 ( RFC2109 )
         // RFC2965 is not supported by browsers and the Servlet spec
