@@ -731,21 +731,7 @@ public class AjpAprProcessor implements ActionHook {
                 String n = tmpMB.toString();
                 requestHeaderMessage.getBytes(tmpMB);
                 String v = tmpMB.toString();
-                /*
-                 * AJP13 misses to forward the remotePort.
-                 * Allow the AJP connector to add this info via
-                 * a private request attribute.
-                 * We will accept the forwarded data as the remote port,
-                 * and remove it from the public list of request attributes.
-                 */
-                if(n.equals(Constants.SC_A_REQ_REMOTE_PORT)) {
-                    try {
-                        request.setRemotePort(Integer.parseInt(v));
-                    } catch (NumberFormatException nfe) {
-                    }
-                } else {
-                    request.setAttribute(n, v);
-                }
+                request.setAttribute(n, v);
                 break;
 
             case Constants.SC_A_CONTEXT :
@@ -965,10 +951,6 @@ public class AjpAprProcessor implements ActionHook {
         } else {
             message = message.replace('\n', ' ').replace('\r', ' ');
         }
-        if (message == null) {
-            // mod_jk + httpd 2.x fails with a null status message - bug 45026
-            message = Integer.toString(response.getStatus());
-        }
         tmpMB.setString(message);
         responseHeaderMessage.appendBytes(tmpMB);
 
