@@ -20,7 +20,7 @@
  *                                                                         *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 756058 $                                          *
+ * Version:     $Revision: 666959 $                                          *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -44,13 +44,9 @@
 #define JK_UWMAP_EXTENSION_DISABLED       "disabled="
 #define JK_UWMAP_EXTENSION_STOPPED        "stopped="
 #define JK_UWMAP_EXTENSION_FAIL_ON_STATUS "fail_on_status="
-#define JK_UWMAP_EXTENSION_USE_SRV_ERRORS "use_server_errors="
 
-#define IND_SWITCH(x)                      (((x)+1) % 2)
 #define IND_THIS(x)                        ((x)[uw_map->index])
-#define IND_NEXT(x)                        ((x)[IND_SWITCH(uw_map->index)])
-
-#define STRNULL_FOR_NULL(x) ((x) ? (x) : "(null)")
+#define IND_NEXT(x)                        ((x)[(uw_map->index+1) % 2])
 
 static const char *uri_worker_map_source_type[] = {
     "unknown",
@@ -159,43 +155,6 @@ static void worker_qsort(jk_uri_worker_map_t *uw_map)
 
 }
 
-/* Dump the map contents - only call if debug log is active. */
-static void uri_worker_map_dump(jk_uri_worker_map_t *uw_map,
-                                const char *reason, jk_logger_t *l)
-{
-    JK_TRACE_ENTER(l);
-    if (uw_map) {
-        int i, off, k;
-        uri_worker_record_t *uwr = NULL;
-        char buf[32];
-        jk_log(l, JK_LOG_DEBUG, "uri map dump %s: index=%d file='%s' reject_unsafe=%d "
-               "reload=%d modified=%d checked=%d",
-               reason, uw_map->index, STRNULL_FOR_NULL(uw_map->fname), uw_map->reject_unsafe,
-               uw_map->reload, uw_map->modified, uw_map->checked);
-        for(i=0;i<=1;i++) {
-            jk_log(l, JK_LOG_DEBUG, "generation %d: size=%d nosize=%d capacity=%d",
-                   i, uw_map->size[i], uw_map->nosize[i], uw_map->capacity[i], uw_map->maps[i]);
-        }
-
-        off = uw_map->index;
-        for(i=0;i<=1;i++) {
-            unsigned int j;
-            k = (i + off) % 2;
-            for (j = 0; j < uw_map->size[k]; j++) {
-                uwr = uw_map->maps[k][j];
-                jk_log(l, JK_LOG_DEBUG, "%s (%d) map #%d: uri=%s worker=%s context=%s "
-                       "source=%s type=%s len=%d",
-                       i ? "NEXT" : "THIS", i, j,
-                       STRNULL_FOR_NULL(uwr->uri), STRNULL_FOR_NULL(uwr->worker_name),
-                       STRNULL_FOR_NULL(uwr->context), STRNULL_FOR_NULL(uri_worker_map_get_source(uwr,l)),
-                       STRNULL_FOR_NULL(uri_worker_map_get_match(uwr,buf,l)), uwr->context_len);
-            }
-        }
-    }
-    JK_TRACE_EXIT(l);
-}
-
-
 int uri_worker_map_alloc(jk_uri_worker_map_t **uw_map_p,
                          jk_map_t *init_data, jk_logger_t *l)
 {
@@ -458,7 +417,7 @@ static void extract_fail_on_status(jk_uri_worker_map_t *uw_map,
     cnt = 0;
 #ifdef _MT_CODE_PTHREAD
     for (status = strtok_r(status, ", ", &lasts);
-         status; status = strtok_r(NULL, ", ", &lasts)) {
+         status; status = strtok_r(NULL, "&", &lasts)) {
 #else
     for (status = strtok(status, ", "); status; status = strtok(NULL, ", ")) {
 #endif
@@ -470,26 +429,6 @@ static void extract_fail_on_status(jk_uri_worker_map_t *uw_map,
 
 }
 
-void uri_worker_map_switch(jk_uri_worker_map_t *uw_map, jk_logger_t *l)
-{
-    int new_index;
-
-    JK_TRACE_ENTER(l);
-
-    if (uw_map) {
-        new_index = IND_SWITCH(uw_map->index);
-        if (JK_IS_DEBUG_LEVEL(l))
-            jk_log(l, JK_LOG_DEBUG,
-                   "Switching uri worker map from index %d to index %d",
-                   uw_map->index, new_index);
-        uw_map->index = new_index;
-        jk_reset_pool(&(IND_NEXT(uw_map->p_dyn)));
-    }
-
-    JK_TRACE_EXIT(l);
-
-}
-
 void uri_worker_map_ext(jk_uri_worker_map_t *uw_map, jk_logger_t *l)
 {
     unsigned int i;
@@ -570,15 +509,14 @@ void uri_worker_map_ext(jk_uri_worker_map_t *uw_map, jk_logger_t *l)
             extract_fail_on_status(uw_map, uwr, l);
         }
     }
-    if (JK_IS_DEBUG_LEVEL(l))
-        uri_worker_map_dump(uw_map, "after extension stripping", l);
+    uw_map->index = (uw_map->index + 1) % 2;
+    jk_reset_pool(&(IND_NEXT(uw_map->p_dyn)));
 
     JK_TRACE_EXIT(l);
     return;
 
 }
 
-/* Add new entry to NEXT generation */
 int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
                        const char *puri, const char *worker,
                        unsigned int source_type, jk_logger_t *l)
@@ -645,7 +583,6 @@ int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
         uwr->extensions.fail_on_status_size = 0;
         uwr->extensions.fail_on_status = NULL;
         uwr->extensions.fail_on_status_str = NULL;
-        uwr->extensions.use_server_error_pages = 0;
 
 #ifdef _MT_CODE_PTHREAD
         param = strtok_r(w, ";", &lasts);
@@ -661,9 +598,6 @@ int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
                 if (!strncmp(param, JK_UWMAP_EXTENSION_REPLY_TIMEOUT, strlen(JK_UWMAP_EXTENSION_REPLY_TIMEOUT))) {
                     uwr->extensions.reply_timeout = atoi(param + strlen(JK_UWMAP_EXTENSION_REPLY_TIMEOUT));
                 }
-                else if (!strncmp(param, JK_UWMAP_EXTENSION_USE_SRV_ERRORS, strlen(JK_UWMAP_EXTENSION_USE_SRV_ERRORS))) {
-                    uwr->extensions.use_server_error_pages = atoi(param + strlen(JK_UWMAP_EXTENSION_USE_SRV_ERRORS));
-                }
                 else if (!strncmp(param, JK_UWMAP_EXTENSION_ACTIVE, strlen(JK_UWMAP_EXTENSION_ACTIVE))) {
                     if (uwr->extensions.active)
                         jk_log(l, JK_LOG_WARNING,
@@ -692,7 +626,7 @@ int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
                     if (uwr->extensions.fail_on_status_str)
                         jk_log(l, JK_LOG_WARNING,
                                "extension '%s' in uri worker map only allowed once",
-                               JK_UWMAP_EXTENSION_FAIL_ON_STATUS);
+                               JK_UWMAP_EXTENSION_ACTIVE);
                     else
                         uwr->extensions.fail_on_status_str = param + strlen(JK_UWMAP_EXTENSION_FAIL_ON_STATUS);
                 }
@@ -823,8 +757,6 @@ int uri_worker_map_open(jk_uri_worker_map_t *uw_map,
             jk_close_pool(&uw_map->p_dyn[1]);
             jk_close_pool(&uw_map->p);
         }
-        else if (JK_IS_DEBUG_LEVEL(l))
-            uri_worker_map_dump(uw_map, "after map open", l);
     }
 
     JK_TRACE_EXIT(l);
@@ -939,7 +871,6 @@ static int is_nomatch(jk_uri_worker_map_t *uw_map,
 const char *map_uri_to_worker_ext(jk_uri_worker_map_t *uw_map,
                                   const char *uri, const char *vhost,
                                   rule_extension_t **extensions,
-                                  int *index,
                                   jk_logger_t *l)
 {
     unsigned int i;
@@ -956,8 +887,6 @@ const char *map_uri_to_worker_ext(jk_uri_worker_map_t *uw_map,
         return NULL;
     }
     *extensions = NULL;
-    if (index)
-        *index = -1;
     if (*uri != '/') {
         jk_log(l, JK_LOG_WARNING,
                 "Uri %s is invalid. Uri must start with /", uri);
@@ -979,19 +908,19 @@ const char *map_uri_to_worker_ext(jk_uri_worker_map_t *uw_map,
      */
     reject_unsafe = uw_map->reject_unsafe;
     vhost_len = 0;
-    /*
-     * In case we got a vhost, we prepend a slash
-     * and the vhost to the url in order to enable
-     * vhost mapping rules especially for IIS.
-     */
+/*
+ * In case we got a vhost, we prepend a slash
+ * and the vhost to the url in order to enable
+ * vhost mapping rules especially for IIS.
+ */
     if (vhost) {
         int off = 0;
-        /* Add leading '/' if necessary */
+/* Add leading '/' if necessary */
         if (vhost[0] != '/') {
             url[0] = '/';
             off = 1;
         }
-        /* Size including leading slash. */
+/* Size including leading slash. */
         vhost_len = strlen(vhost);
         if (vhost_len + off >= JK_MAX_URI_LEN) {
             vhost_len = 0;
@@ -1036,16 +965,16 @@ const char *map_uri_to_worker_ext(jk_uri_worker_map_t *uw_map,
         jk_log(l, JK_LOG_DEBUG, "Attempting to map URI '%s' from %d maps",
                url, IND_THIS(uw_map->size));
     rv = find_match(uw_map, url, l);
-    /* If this doesn't find a match, try without the vhost. */
+/* If this doesn't find a match, try without the vhost. */
     if (rv < 0 && vhost_len) {
         rv = find_match(uw_map, &url[vhost_len], l);
     }
 
-    /* In case we found a match, check for the unmounts. */
+/* In case we found a match, check for the unmounts. */
     if (rv >= 0 && IND_THIS(uw_map->nosize)) {
-        /* Again first including vhost. */
+/* Again first including vhost. */
         int rc = is_nomatch(uw_map, url, rv, l);
-        /* If no unmount was find, try without vhost. */
+/* If no unmount was find, try without vhost. */
         if (!rc && vhost_len)
             rc = is_nomatch(uw_map, &url[vhost_len], rv, l);
         if (rc) {
@@ -1060,8 +989,6 @@ const char *map_uri_to_worker_ext(jk_uri_worker_map_t *uw_map,
 
     if (rv >= 0) {
         *extensions = &(IND_THIS(uw_map->maps)[rv]->extensions);
-        if (index)
-            *index = rv;
         JK_TRACE_EXIT(l);
         return IND_THIS(uw_map->maps)[rv]->worker_name;
     }
@@ -1069,23 +996,12 @@ const char *map_uri_to_worker_ext(jk_uri_worker_map_t *uw_map,
     return NULL;
 }
 
-rule_extension_t *get_uri_to_worker_ext(jk_uri_worker_map_t *uw_map,
-                                        int index)
-{
-    if (index >= 0) {
-        return &(IND_THIS(uw_map->maps)[index]->extensions);
-    }
-    else {
-        return NULL;
-    }
-}
-
 const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
                               const char *uri, const char *vhost,
                               jk_logger_t *l)
 {
     rule_extension_t *ext;
-    return map_uri_to_worker_ext(uw_map, uri, vhost, &ext, NULL, l);
+    return map_uri_to_worker_ext(uw_map, uri, vhost, &ext, l);
 }
 
 int uri_worker_map_load(jk_uri_worker_map_t *uw_map,
@@ -1095,7 +1011,7 @@ int uri_worker_map_load(jk_uri_worker_map_t *uw_map,
     jk_map_t *map;
 
     jk_map_alloc(&map);
-    if (jk_map_read_properties(map, NULL, uw_map->fname, &uw_map->modified,
+    if (jk_map_read_properties(map, uw_map->fname, &uw_map->modified,
                                JK_MAP_HANDLE_NORMAL, l)) {
         int i;
         if (JK_IS_DEBUG_LEVEL(l))
@@ -1139,8 +1055,6 @@ int uri_worker_map_load(jk_uri_worker_map_t *uw_map,
             }
         }
         uw_map->checked = time(NULL);
-        if (JK_IS_DEBUG_LEVEL(l))
-            uri_worker_map_dump(uw_map, "after file load", l);
         rc = JK_TRUE;
     }
     jk_map_free(&map);
@@ -1153,8 +1067,8 @@ int uri_worker_map_update(jk_uri_worker_map_t *uw_map,
     int rc = JK_TRUE;
     time_t now = time(NULL);
 
-    if (force || (uw_map->reload > 0 && difftime(now, uw_map->checked) >
-                  uw_map->reload)) {
+    if ((uw_map->reload > 0 && difftime(now, uw_map->checked) > uw_map->reload) ||
+        force) {
         struct stat statbuf;
         uw_map->checked = now;
         if ((rc = jk_stat(uw_map->fname, &statbuf)) == -1) {
@@ -1182,7 +1096,6 @@ int uri_worker_map_update(jk_uri_worker_map_t *uw_map,
         }
         rc = uri_worker_map_load(uw_map, l);
         uri_worker_map_ext(uw_map, l);
-        uri_worker_map_switch(uw_map, l);
         JK_LEAVE_CS(&(uw_map->cs), rc);
         jk_log(l, JK_LOG_INFO,
                "Reloaded urimaps from %s", uw_map->fname);
