@@ -235,7 +235,7 @@ import org.apache.catalina.util.IOTools;
  *
  * @author Martin T Dengler [root@martindengler.com]
  * @author Amy Roh
- * @version $Revision: 505740 $, $Date: 2007-02-10 11:45:24 -0700 (Sat, 10 Feb 2007) $
+ * @version $Revision: 681141 $, $Date: 2008-07-30 11:46:08 -0600 (Wed, 30 Jul 2008) $
  * @since Tomcat 4.0
  *
  */
@@ -667,9 +667,7 @@ public final class CGIServlet extends HttpServlet {
         if (OS.indexOf("windows 9") > -1) {
             p = r.exec( "command.com /c set" );
             ignoreCase = true;
-        } else if ( (OS.indexOf("nt") > -1)
-                || (OS.indexOf("windows 20") > -1)
-                || (OS.indexOf("windows xp") > -1) ) {
+        } else if ((OS.indexOf("nt") > -1) || (OS.indexOf("windows") > -1) ) {
             // thanks to JuanFran for the xp fix!
             p = r.exec( "cmd.exe /c set" );
             ignoreCase = true;
@@ -707,7 +705,7 @@ public final class CGIServlet extends HttpServlet {
      * <p>
      * </p>
      *
-     * @version  $Revision: 505740 $, $Date: 2007-02-10 11:45:24 -0700 (Sat, 10 Feb 2007) $
+     * @version  $Revision: 681141 $, $Date: 2008-07-30 11:46:08 -0600 (Wed, 30 Jul 2008) $
      * @since    Tomcat 4.0
      *
      */
@@ -797,10 +795,26 @@ public final class CGIServlet extends HttpServlet {
          */
         protected void setupFromRequest(HttpServletRequest req)
                 throws UnsupportedEncodingException {
-            
-            this.contextPath = req.getContextPath();
-            this.servletPath = req.getServletPath();
-            this.pathInfo = req.getPathInfo();
+
+            boolean isIncluded = false;
+
+            // Look to see if this request is an include
+            if (req.getAttribute(Globals.INCLUDE_REQUEST_URI_ATTR) != null) {
+                isIncluded = true;
+            }
+            if (isIncluded) {
+                this.contextPath = (String) req.getAttribute(
+                        Globals.INCLUDE_CONTEXT_PATH_ATTR);
+                this.servletPath = (String) req.getAttribute(
+                        Globals.INCLUDE_SERVLET_PATH_ATTR);
+                this.pathInfo = (String) req.getAttribute(
+                        Globals.INCLUDE_PATH_INFO_ATTR);
+            } else {
+                this.contextPath = req.getContextPath();
+                this.servletPath = req.getServletPath();
+                this.pathInfo = req.getPathInfo();
+            }
+
             // If getPathInfo() returns null, must be using extension mapping
             // In this case, pathInfo should be same as servletPath
             if (this.pathInfo == null) {
@@ -814,7 +828,13 @@ public final class CGIServlet extends HttpServlet {
             if (req.getMethod().equals("GET")
                 || req.getMethod().equals("POST")
                 || req.getMethod().equals("HEAD")) {
-                String qs = req.getQueryString();
+                String qs;
+                if (isIncluded) {
+                    qs = (String) req.getAttribute(
+                            Globals.INCLUDE_QUERY_STRING_ATTR);
+                } else {
+                    qs = req.getQueryString();
+                }
                 if (qs != null && qs.indexOf("=") == -1) {
                     StringTokenizer qsTokens = new StringTokenizer(qs, "+");
                     while ( qsTokens.hasMoreTokens() ) {
@@ -1429,7 +1449,7 @@ public final class CGIServlet extends HttpServlet {
      * and <code>setResponse</code> methods, respectively.
      * </p>
      *
-     * @version   $Revision: 505740 $, $Date: 2007-02-10 11:45:24 -0700 (Sat, 10 Feb 2007) $
+     * @version   $Revision: 681141 $, $Date: 2008-07-30 11:46:08 -0600 (Wed, 30 Jul 2008) $
      */
 
     protected class CGIRunner {
