@@ -23,9 +23,10 @@ import java.lang.reflect.Method;
 import org.apache.catalina.Context;
 import org.apache.catalina.Wrapper;
 import org.apache.catalina.deploy.SecurityConstraint;
-import org.apache.commons.digester.Digester;
-import org.apache.commons.digester.Rule;
-import org.apache.commons.digester.RuleSetBase;
+import org.apache.catalina.deploy.SecurityRoleRef;
+import org.apache.tomcat.util.digester.Digester;
+import org.apache.tomcat.util.digester.Rule;
+import org.apache.tomcat.util.digester.RuleSetBase;
 import org.xml.sax.Attributes;
 
 
@@ -34,7 +35,7 @@ import org.xml.sax.Attributes;
  * deployment descriptor (<code>/WEB-INF/web.xml</code>) resource.</p>
  *
  * @author Craig R. McClanahan
- * @version $Revision: 466595 $ $Date: 2006-10-21 23:24:41 +0100 (Sat, 21 Oct 2006) $
+ * @version $Revision: 638546 $ $Date: 2008-03-18 13:58:29 -0600 (Tue, 18 Mar 2008) $
  */
 
 public class WebRuleSet extends RuleSetBase {
@@ -47,6 +48,24 @@ public class WebRuleSet extends RuleSetBase {
      * The matching pattern prefix to use for recognizing our elements.
      */
     protected String prefix = null;
+    
+    
+    /**
+     * The <code>SetSessionConfig</code> rule used to parse the web.xml
+     */
+    protected SetSessionConfig sessionConfig;
+    
+    
+    /**
+     * The <code>SetLoginConfig</code> rule used to parse the web.xml
+     */
+    protected SetLoginConfig loginConfig;
+
+    
+    /**
+     * The <code>SetJspConfig</code> rule used to parse the web.xml
+     */    
+    protected SetJspConfig jspConfig;
 
 
     // ------------------------------------------------------------ Constructor
@@ -92,9 +111,12 @@ public class WebRuleSet extends RuleSetBase {
      *  should be added.
      */
     public void addRuleInstances(Digester digester) {
-
+        sessionConfig = new SetSessionConfig();
+        jspConfig = new SetJspConfig();
+        loginConfig = new SetLoginConfig();
+        
         digester.addRule(prefix + "web-app",
-                         new SetPublicIdRule(digester, "setPublicId"));
+                         new SetPublicIdRule("setPublicId"));
 
         digester.addCallMethod(prefix + "web-app/context-param",
                                "addParameter", 2);
@@ -105,13 +127,13 @@ public class WebRuleSet extends RuleSetBase {
                                "setDisplayName", 0);
 
         digester.addRule(prefix + "web-app/distributable",
-                         new SetDistributableRule(digester));
+                         new SetDistributableRule());
 
         digester.addObjectCreate(prefix + "web-app/ejb-local-ref",
                                  "org.apache.catalina.deploy.ContextLocalEjb");
-        digester.addSetNext(prefix + "web-app/ejb-local-ref",
-                            "addLocalEjb",
-                            "org.apache.catalina.deploy.ContextLocalEjb");
+        digester.addRule(prefix + "web-app/ejb-local-ref",
+                new SetNextNamingRule("addLocalEjb",
+                            "org.apache.catalina.deploy.ContextLocalEjb"));
 
         digester.addCallMethod(prefix + "web-app/ejb-local-ref/description",
                                "setDescription", 0);
@@ -128,9 +150,9 @@ public class WebRuleSet extends RuleSetBase {
 
         digester.addObjectCreate(prefix + "web-app/ejb-ref",
                                  "org.apache.catalina.deploy.ContextEjb");
-        digester.addSetNext(prefix + "web-app/ejb-ref",
-                            "addEjb",
-                            "org.apache.catalina.deploy.ContextEjb");
+        digester.addRule(prefix + "web-app/ejb-ref",
+                new SetNextNamingRule("addEjb",
+                            "org.apache.catalina.deploy.ContextEjb"));
 
         digester.addCallMethod(prefix + "web-app/ejb-ref/description",
                                "setDescription", 0);
@@ -147,9 +169,9 @@ public class WebRuleSet extends RuleSetBase {
 
         digester.addObjectCreate(prefix + "web-app/env-entry",
                                  "org.apache.catalina.deploy.ContextEnvironment");
-        digester.addSetNext(prefix + "web-app/env-entry",
-                            "addEnvironment",
-                            "org.apache.catalina.deploy.ContextEnvironment");
+        digester.addRule(prefix + "web-app/env-entry",
+                new SetNextNamingRule("addEnvironment",
+                            "org.apache.catalina.deploy.ContextEnvironment"));
 
         digester.addCallMethod(prefix + "web-app/env-entry/description",
                                "setDescription", 0);
@@ -212,8 +234,20 @@ public class WebRuleSet extends RuleSetBase {
         digester.addCallMethod(prefix + "web-app/filter-mapping/url-pattern",
                                "setURLPattern", 0);
 
-        digester.addCallMethod(prefix + "web-app/listener/listener-class",
-                               "addApplicationListener", 0);
+        digester.addCallMethod(prefix + "web-app/filter-mapping/dispatcher",
+                               "setDispatcher", 0);
+
+         digester.addCallMethod(prefix + "web-app/listener/listener-class",
+                                "addApplicationListener", 0);
+         
+        digester.addRule(prefix + "web-app/jsp-config",
+                         jspConfig);
+        
+        digester.addCallMethod(prefix + "web-app/jsp-config/jsp-property-group/url-pattern",
+                               "addJspMapping", 0);
+
+        digester.addRule(prefix + "web-app/login-config",
+                         loginConfig);
 
         digester.addObjectCreate(prefix + "web-app/login-config",
                                  "org.apache.catalina.deploy.LoginConfig");
@@ -235,16 +269,56 @@ public class WebRuleSet extends RuleSetBase {
         digester.addCallParam(prefix + "web-app/mime-mapping/extension", 0);
         digester.addCallParam(prefix + "web-app/mime-mapping/mime-type", 1);
 
-        digester.addCallMethod(prefix + "web-app/resource-env-ref",
-                               "addResourceEnvRef", 2);
-        digester.addCallParam(prefix + "web-app/resource-env-ref/resource-env-ref-name", 0);
-        digester.addCallParam(prefix + "web-app/resource-env-ref/resource-env-ref-type", 1);
+        digester.addObjectCreate(prefix + "web-app/resource-env-ref",
+            "org.apache.catalina.deploy.ContextResourceEnvRef");
+        digester.addRule(prefix + "web-app/resource-env-ref",
+                    new SetNextNamingRule("addResourceEnvRef",
+                        "org.apache.catalina.deploy.ContextResourceEnvRef"));
+
+        digester.addCallMethod(prefix + "web-app/resource-env-ref/resource-env-ref-name",
+                "setName", 0);
+        digester.addCallMethod(prefix + "web-app/resource-env-ref/resource-env-ref-type",
+                "setType", 0);
+
+        digester.addObjectCreate(prefix + "web-app/message-destination",
+                                 "org.apache.catalina.deploy.MessageDestination");
+        digester.addSetNext(prefix + "web-app/message-destination",
+                            "addMessageDestination",
+                            "org.apache.catalina.deploy.MessageDestination");
+
+        digester.addCallMethod(prefix + "web-app/message-destination/description",
+                               "setDescription", 0);
+        digester.addCallMethod(prefix + "web-app/message-destination/display-name",
+                               "setDisplayName", 0);
+        digester.addCallMethod(prefix + "web-app/message-destination/icon/large-icon",
+                               "setLargeIcon", 0);
+        digester.addCallMethod(prefix + "web-app/message-destination/icon/small-icon",
+                               "setSmallIcon", 0);
+        digester.addCallMethod(prefix + "web-app/message-destination/message-destination-name",
+                               "setName", 0);
+
+        digester.addObjectCreate(prefix + "web-app/message-destination-ref",
+                                 "org.apache.catalina.deploy.MessageDestinationRef");
+        digester.addSetNext(prefix + "web-app/message-destination-ref",
+                            "addMessageDestinationRef",
+                            "org.apache.catalina.deploy.MessageDestinationRef");
+
+        digester.addCallMethod(prefix + "web-app/message-destination-ref/description",
+                               "setDescription", 0);
+        digester.addCallMethod(prefix + "web-app/message-destination-ref/message-destination-link",
+                               "setLink", 0);
+        digester.addCallMethod(prefix + "web-app/message-destination-ref/message-destination-ref-name",
+                               "setName", 0);
+        digester.addCallMethod(prefix + "web-app/message-destination-ref/message-destination-type",
+                               "setType", 0);
+        digester.addCallMethod(prefix + "web-app/message-destination-ref/message-destination-usage",
+                               "setUsage", 0);
 
         digester.addObjectCreate(prefix + "web-app/resource-ref",
                                  "org.apache.catalina.deploy.ContextResource");
-        digester.addSetNext(prefix + "web-app/resource-ref",
-                            "addResource",
-                            "org.apache.catalina.deploy.ContextResource");
+        digester.addRule(prefix + "web-app/resource-ref",
+                new SetNextNamingRule("addResource",
+                            "org.apache.catalina.deploy.ContextResource"));
 
         digester.addCallMethod(prefix + "web-app/resource-ref/description",
                                "setDescription", 0);
@@ -264,7 +338,7 @@ public class WebRuleSet extends RuleSetBase {
                             "org.apache.catalina.deploy.SecurityConstraint");
 
         digester.addRule(prefix + "web-app/security-constraint/auth-constraint",
-                         new SetAuthConstraintRule(digester));
+                         new SetAuthConstraintRule());
         digester.addCallMethod(prefix + "web-app/security-constraint/auth-constraint/role-name",
                                "addAuthRole", 0);
         digester.addCallMethod(prefix + "web-app/security-constraint/display-name",
@@ -288,7 +362,7 @@ public class WebRuleSet extends RuleSetBase {
                                "addSecurityRole", 0);
 
         digester.addRule(prefix + "web-app/servlet",
-                         new WrapperCreateRule(digester));
+                         new WrapperCreateRule());
         digester.addSetNext(prefix + "web-app/servlet",
                             "addChild",
                             "org.apache.catalina.Container");
@@ -307,10 +381,15 @@ public class WebRuleSet extends RuleSetBase {
         digester.addCallMethod(prefix + "web-app/servlet/run-as/role-name",
                                "setRunAs", 0);
 
-        digester.addCallMethod(prefix + "web-app/servlet/security-role-ref",
-                               "addSecurityReference", 2);
-        digester.addCallParam(prefix + "web-app/servlet/security-role-ref/role-link", 1);
-        digester.addCallParam(prefix + "web-app/servlet/security-role-ref/role-name", 0);
+        digester.addRule(prefix + "web-app/servlet/security-role-ref",
+                new SecurityRoleRefCreateRule());
+
+        digester.addCallMethod(
+                prefix + "web-app/servlet/security-role-ref/role-link",
+                "setLink", 0);
+        digester.addCallMethod(
+                prefix + "web-app/servlet/security-role-ref/role-name",
+                "setName", 0);
 
         digester.addCallMethod(prefix + "web-app/servlet/servlet-class",
                               "setServletClass", 0);
@@ -322,6 +401,9 @@ public class WebRuleSet extends RuleSetBase {
         digester.addCallParam(prefix + "web-app/servlet-mapping/servlet-name", 1);
         digester.addCallParam(prefix + "web-app/servlet-mapping/url-pattern", 0);
 
+        digester.addRule(prefix + "web-app/session-config",
+                         sessionConfig);
+        
         digester.addCallMethod(prefix + "web-app/session-config/session-timeout",
                                "setSessionTimeout", 1,
                                new Class[] { Integer.TYPE });
@@ -335,15 +417,89 @@ public class WebRuleSet extends RuleSetBase {
         digester.addCallMethod(prefix + "web-app/welcome-file-list/welcome-file",
                                "addWelcomeFile", 0);
 
+        digester.addCallMethod(prefix + "web-app/locale-encoding-mapping-list/locale-encoding-mapping",
+                              "addLocaleEncodingMappingParameter", 2);
+        digester.addCallParam(prefix + "web-app/locale-encoding-mapping-list/locale-encoding-mapping/locale", 0);
+        digester.addCallParam(prefix + "web-app/locale-encoding-mapping-list/locale-encoding-mapping/encoding", 1);
+
     }
 
-
+    /**
+     * Reset counter used for validating the web.xml file.
+     */
+    public void recycle(){
+        jspConfig.isJspConfigSet = false;
+        sessionConfig.isSessionConfigSet = false;
+        loginConfig.isLoginConfigSet = false;
+    }
 }
 
 
 // ----------------------------------------------------------- Private Classes
 
 
+/**
+ * Rule to check that the <code>login-config</code> is occuring 
+ * only 1 time within the web.xml
+ */
+final class SetLoginConfig extends Rule {
+    protected boolean isLoginConfigSet = false;
+    public SetLoginConfig() {
+    }
+
+    public void begin(String namespace, String name, Attributes attributes)
+        throws Exception {
+        if (isLoginConfigSet){
+            throw new IllegalArgumentException(
+            "<login-config> element is limited to 1 occurance");
+        }
+        isLoginConfigSet = true;
+    }
+
+}
+
+
+/**
+ * Rule to check that the <code>jsp-config</code> is occuring 
+ * only 1 time within the web.xml
+ */
+final class SetJspConfig extends Rule {
+    protected boolean isJspConfigSet = false;
+    public SetJspConfig() {
+    }
+
+    public void begin(String namespace, String name, Attributes attributes)
+        throws Exception {
+        if (isJspConfigSet){
+            throw new IllegalArgumentException(
+            "<jsp-config> element is limited to 1 occurance");
+        }
+        isJspConfigSet = true;
+    }
+
+}
+
+
+/**
+ * Rule to check that the <code>session-config</code> is occuring 
+ * only 1 time within the web.xml
+ */
+final class SetSessionConfig extends Rule {
+    protected boolean isSessionConfigSet = false;
+    public SetSessionConfig() {
+    }
+
+    public void begin(String namespace, String name, Attributes attributes)
+        throws Exception {
+        if (isSessionConfigSet){
+            throw new IllegalArgumentException(
+            "<session-config> element is limited to 1 occurance");
+        }
+        isSessionConfigSet = true;
+    }
+
+}
+
 /**
  * A Rule that calls the <code>setAuthConstraint(true)</code> method of
  * the top item on the stack, which must be of type
@@ -352,16 +508,18 @@ public class WebRuleSet extends RuleSetBase {
 
 final class SetAuthConstraintRule extends Rule {
 
-    public SetAuthConstraintRule(Digester digester) {
-        super(digester);
+    public SetAuthConstraintRule() {
     }
 
-    public void begin(Attributes attributes) throws Exception {
+    public void begin(String namespace, String name, Attributes attributes)
+        throws Exception {
         SecurityConstraint securityConstraint =
             (SecurityConstraint) digester.peek();
         securityConstraint.setAuthConstraint(true);
-        if (digester.getDebug() > 0)
-            digester.log("Calling SecurityConstraint.setAuthConstraint(true)");
+        if (digester.getLogger().isDebugEnabled()) {
+            digester.getLogger()
+               .debug("Calling SecurityConstraint.setAuthConstraint(true)");
+        }
     }
 
 }
@@ -374,16 +532,17 @@ final class SetAuthConstraintRule extends Rule {
 
 final class SetDistributableRule extends Rule {
 
-    public SetDistributableRule(Digester digester) {
-        super(digester);
+    public SetDistributableRule() {
     }
 
-    public void begin(Attributes attributes) throws Exception {
+    public void begin(String namespace, String name, Attributes attributes)
+        throws Exception {
         Context context = (Context) digester.peek();
         context.setDistributable(true);
-        if (digester.getDebug() > 0)
-            digester.log(context.getClass().getName() +
-                         ".setDistributable( true)");
+        if (digester.getLogger().isDebugEnabled()) {
+            digester.getLogger().debug
+               (context.getClass().getName() + ".setDistributable( true)");
+        }
     }
 
 }
@@ -396,14 +555,14 @@ final class SetDistributableRule extends Rule {
 
 final class SetPublicIdRule extends Rule {
 
-    public SetPublicIdRule(Digester digester, String method) {
-        super(digester);
+    public SetPublicIdRule(String method) {
         this.method = method;
     }
 
     private String method = null;
 
-    public void begin(Attributes attributes) throws Exception {
+    public void begin(String namespace, String name, Attributes attributes)
+        throws Exception {
 
         Object top = digester.peek();
         Class paramClasses[] = new Class[1];
@@ -415,15 +574,15 @@ final class SetPublicIdRule extends Rule {
         try {
             m = top.getClass().getMethod(method, paramClasses);
         } catch (NoSuchMethodException e) {
-            digester.log("Can't find method " + method + " in " + top +
-                         " CLASS " + top.getClass());
+            digester.getLogger().error("Can't find method " + method + " in "
+                                       + top + " CLASS " + top.getClass());
             return;
         }
 
-        m.invoke(top, paramValues);
-        if (digester.getDebug() >= 1)
-            digester.log("" + top.getClass().getName() + "." + method +
-                        "(" + paramValues[0] + ")");
+        m.invoke(top, (Object [])paramValues);
+        if (digester.getLogger().isDebugEnabled())
+            digester.getLogger().debug("" + top.getClass().getName() + "." 
+                                       + method + "(" + paramValues[0] + ")");
 
     }
 
@@ -437,23 +596,54 @@ final class SetPublicIdRule extends Rule {
 
 final class WrapperCreateRule extends Rule {
 
-    public WrapperCreateRule(Digester digester) {
-        super(digester);
+    public WrapperCreateRule() {
     }
 
-    public void begin(Attributes attributes) throws Exception {
+    public void begin(String namespace, String name, Attributes attributes)
+        throws Exception {
         Context context =
             (Context) digester.peek(digester.getCount() - 1);
         Wrapper wrapper = context.createWrapper();
         digester.push(wrapper);
-        if (digester.getDebug() > 0)
-            digester.log("new " + wrapper.getClass().getName());
+        if (digester.getLogger().isDebugEnabled())
+            digester.getLogger().debug("new " + wrapper.getClass().getName());
     }
 
-    public void end() throws Exception {
+    public void end(String namespace, String name)
+        throws Exception {
         Wrapper wrapper = (Wrapper) digester.pop();
-        if (digester.getDebug() > 0)
-            digester.log("pop " + wrapper.getClass().getName());
+        if (digester.getLogger().isDebugEnabled())
+            digester.getLogger().debug("pop " + wrapper.getClass().getName());
     }
 
 }
+
+/**
+ * A Rule that adds a security-role-ref to a servlet, allowing for the fact that
+ * role-link is an optional element.
+ */
+final class SecurityRoleRefCreateRule extends Rule {
+    
+    public SecurityRoleRefCreateRule() {
+    }
+    
+    public void begin(String namespace, String name, Attributes attributes)
+            throws Exception {
+        SecurityRoleRef securityRoleRef = new SecurityRoleRef();
+        digester.push(securityRoleRef);
+        if (digester.getLogger().isDebugEnabled())
+            digester.getLogger().debug("new SecurityRoleRef");
+    }
+
+    public void end(String namespace, String name)
+            throws Exception {
+        SecurityRoleRef securityRoleRef = (SecurityRoleRef) digester.pop();
+        Wrapper wrapper = (Wrapper) digester.peek();
+        
+        wrapper.addSecurityReference(securityRoleRef.getName(),
+                securityRoleRef.getLink());
+
+        if (digester.getLogger().isDebugEnabled())
+            digester.getLogger().debug("pop SecurityRoleRef");
+    }
+}
