@@ -25,6 +25,7 @@ import java.util.HashMap;
 import javax.management.MBeanServer;
 import javax.management.ObjectName;
 
+import org.apache.catalina.Container;
 import org.apache.catalina.Context;
 import org.apache.catalina.Engine;
 import org.apache.catalina.Host;
@@ -53,7 +54,7 @@ import org.apache.commons.modeler.Registry;
  * 
  * @author Filip Hanik
  * @author Peter Rossbach
- * @version $Revision: 466608 $
+ * @version $Revision: 613788 $
  */
 public class FarmWarDeployer implements ClusterDeployer, FileChangeListener {
     /*--Static Variables----------------------------------------*/
@@ -137,23 +138,36 @@ public class FarmWarDeployer implements ClusterDeployer, FileChangeListener {
     public void start() throws Exception {
         if (started)
             return;
-        getCluster().addClusterListener(this);
+        Container hcontainer = getCluster().getContainer();
+        if(!(hcontainer instanceof Host)) {
+            log.error("FarmWarDeployer can only work as host cluster subelement!");
+            return ;
+        }
+        // Check to correct engine and host setup
+        host = (Host) hcontainer;
+        Container econtainer = host.getParent();
+        if(econtainer == null && econtainer instanceof Engine) {
+            log.error("FarmWarDeployer can only work if parent of " + host.getName()+ " is an engine!"); 
+            return ;
+        }
+        Engine engine = (Engine) econtainer;
+        try {
+            oname = new ObjectName(engine.getName() + ":type=Deployer,host="
+                    + host.getName());
+        } catch (Exception e) {
+            log.error("Can't construct MBean object name" + e);
+            return ;
+        }
+        
         if (watchEnabled) {
             watcher = new WarWatcher(this, new File(getWatchDir()));
             if (log.isInfoEnabled())
                 log.info("Cluster deployment is watching " + getWatchDir()
                         + " for changes.");
         }
-
-        // Check to correct engine and host setup
-        host = (Host) getCluster().getContainer();
-        Engine engine = (Engine) host.getParent();
-        try {
-            oname = new ObjectName(engine.getName() + ":type=Deployer,host="
-                    + host.getName());
-        } catch (Exception e) {
-            log.error("Can't construct MBean object name" + e);
-        }
+        
+        getCluster().addClusterListener(this);
+        
         configBase = new File(System.getProperty("catalina.base"), "conf");
         if (engine != null) {
             configBase = new File(configBase, engine.getName());
